/*********************************************
 * OPL 12.6.1.0 Model
 * Author: rpilling
 * Creation Date: 20/04/2015 at 11:10:05 PM
 *********************************************/

//declare sets
{string} DG = ...;
{string} UG = ...;
int NbPeriods = ...;
range Periods = 1..NbPeriods;

// declare microgrid input variables 
float aM[DG] = ...;
float bM[DG]= ...;
float cM[DG]= ...;
float minpowerM[DG] = ...;
float maxpowerM[DG] = ...;

float aU[UG] = ...;
float bU[UG]= ...;
float cU[UG]= ...;

float demandM[Periods] = ...;
float demandU[Periods] = ...;
float PVout[Periods] = ...;

// declare microgrid decision variables
dvar float+ xM[DG][Periods];
dvar float+ xU[UG][Periods];
dvar int m[Periods] in 0..1;

//  objective function (capacity)
minimize
  sum(g in DG, t in Periods)(aM[g]*xM[g][t]^2 + bM[g]*xM[g][t] + m[t]*cM[g])+ sum(k in UG, t in Periods)(aU[k]*xU[k][t]^2 - bU[k]*xU[k][t] + cU[k]);
    
// constraint equations        
subject to {

// global power balance constraint 
forall (g in DG, t in Periods, k in UG)
  ctDemand: xM[g][t]+ xU[k][t] == demandM[t] + demandU[t] - PVout[t];// balance equation
  
// DG power limit constraints
forall (g in DG, t in Periods)
  ctDGLowerLimit: xM[g][t] >= minpowerM[g];

forall (g in DG, t in Periods)
  ctDGUpperLimit: xM[g][t] <= maxpowerM[g];
  
  // logical constraint
forall (g in DG, t in Periods) xM[g][t]>=1/1000 => m[t]==1;
}