/*********************************************
 * OPL 12.6.1.0 Model
 * Author: rpilling
 * Creation Date: 20/04/2015 at 11:10:05 PM
 *********************************************/

//declare sets
{string} DG = ...;
{string} UG = ...;
{string} ESS = ...;

int NbPeriods = ...;
range Periods = 1..NbPeriods;

// declare input variables 
float aM[DG] = ...;
float bM[DG]= ...;
float cM[DG]= ...;
float minpowerM[DG] = ...;
float maxpowerM[DG] = ...;

float aU[UG] = ...;
float bU[UG]= ...;
float cU[UG]= ...;

float demandM[Periods] = ...;
float demandU[Periods] = ...;
float PVout[Periods] = ...;

// declare storage specific variables 
float fcS[ESS] = ...;
float maxin[ESS] = ...;
float maxout[ESS] = ...;
float eff_c[ESS] = ...;
float eff_d[ESS] = ...;
float ceS[ESS] = ...;

// declare decision variables
dvar float+ xM[DG][Periods];
dvar float+ xU[UG][Periods];
dvar int m[Periods] in 0..1;

// declare ESS decision variables
dvar float+ eS[ESS][Periods];
dvar float+ inS[ESS][Periods];
dvar float+ outS[ESS][Periods];
dvar float xS[Periods];
dvar int nS[ESS];

//  objective function (capacity)
minimize
  sum(g in DG, t in Periods)(aM[g]*xM[g][t]^2 + bM[g]*xM[g][t] + m[t]*cM[g])+ sum(k in UG, t in Periods)(aU[k]*xU[k][t]^2 - bU[k]*xU[k][t] + cU[k]);
    
// constraint equations        
subject to {
  
// global power balance constraint 
forall (g in DG, t in Periods, k in UG)
  ctDemand: xM[g][t]+ xU[k][t] + xS[t] == demandM[t] + demandU[t] - PVout[t];// balance equation
  
forall (t in Periods)
  eSOut: sum(s in ESS)(outS[s][t]- inS[s][t]) == xS[t];
  
// DG power limit constraints
forall (g in DG, t in Periods)
  ctDGLowerLimit: xM[g][t] >= minpowerM[g];

forall (g in DG, t in Periods)
  ctDGUpperLimit: xM[g][t] <= maxpowerM[g];

// storage-specific constraints
forall (s in ESS, t in 2..NbPeriods) // storage exceeds output
  ctStorage: eS[s][t-1] >= outS[s][t];

forall (s in ESS) // input-start constraint
  ctInput: eS[s][1] <= inS[s][1];
  
forall (s in ESS, t in 2..NbPeriods)
  ctDBalance: eS[s][t] == eS[s][t-1]-(outS[s][t-1]/eff_d[s])+(inS[s][t-1]*eff_c[s]); //Energy charge balance equation
  
forall (s in ESS, t in Periods)
  ctESSLimit: eS[s][t] <= nS[s]*ceS[s]; // energy ESS limit

forall (s in ESS, t in Periods)
  ctChargeLimit: inS[s][t] <= nS[s]*maxin[s]; // charging limit

forall (s in ESS, t in Periods)
  ctDechargeLimit: outS[s][t] <= nS[s]*maxout[s]; // discharging limit
 
 forall (s in ESS)							// terminal equality constraint
  ctInitialPeriod: eS[s][1] == eS[s][24]-(outS[s][24]/eff_d[s])+(inS[s][24]*eff_c[s]);

 forall (s in ESS)
   EVPolicy: nS[s] <= 100;
   
// logical constraint
forall (g in DG, t in Periods) xM[g][t]>=1/1000 => m[t]==1;
}