/*********************************************
 * OPL 12.6.1.0 Model
 * Author: rpilling
 * Creation Date: 20/04/2015 at 11:10:05 PM
 *********************************************/

//declare sets
{string} DG = ...;
int NbPeriods = ...;
range Periods = 1..NbPeriods;

// declare microgrid input variables 
float A[DG] = ...;
float B[DG]= ...;
float C[DG]= ...;
float minpowerM[DG] = ...;
float maxpowerM[DG] = ...;

float demandM[Periods] = ...;
float PVout[Periods] = ...;

// declare microgrid decision variables
dvar float+ xM[DG][Periods];
dvar int m[Periods] in 0..1; 

//  objective function (capacity)
minimize
  sum(k in DG, t in Periods)(A[k]*(xM[k][t])^2 + B[k]*xM[k][t] + m[t]*C[k]);
  
// constraint equations        
subject to {

// global power balance constraint 
forall (t in Periods)
  ctDemand: sum(k in DG)xM[k][t] >= demandM[t] - PVout[t];// balance equation

// DG power limit constraints
forall (k in DG, t in Periods)
  ctDGLowerLimit: xM[k][t] >= minpowerM[k];

forall (k in DG, t in Periods)
  ctDGUpperLimit: xM[k][t] <= maxpowerM[k];
  
// logical constraint
forall (t in Periods) demandM[t] - PVout[t] >=1/10000 => m[t]==1; 

}